/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using ICSharpCode.SharpZipLib.Zip;
using ICSharpCode.SharpZipLib.Tar;

namespace SDC.Net {
    public class CompressedFile {
        private static String[] ZipExtensions = { ".zip", ".jar"};
        private static String[] RarExtensions = { ".rar"};
        private static String[] TarExtensions = { ".tar" };
        private static String[] SevenZipExtensions = { ".7z" };
        
        public static bool isCompressedFile(String fileName) {

            if (isZipFile(fileName) || isRarFile(fileName) || isTarFile(fileName) || isSevenZipFile(fileName))
                return true;

            return false;
        }

        private static bool isRarFile(String fileName) {
            String strExt = Path.GetExtension(fileName);

            foreach (String s in RarExtensions) {
                if (s == strExt.ToLower()) {
                    return true;
                }
            }

            return false;
        }

        private static bool isTarFile(String fileName) {
            String strExt = Path.GetExtension(fileName);

            foreach (String s in TarExtensions) {
                if (s == strExt.ToLower()) {
                    return true;
                }
            }

            return false;
        }

        private static bool isZipFile(String fileName) {
            String strExt = Path.GetExtension(fileName);

            foreach (String s in ZipExtensions) {
                if (s == strExt.ToLower()) {
                    return true;
                }
            }

            return false;
        }

        public static bool isSevenZipFile(String fileName)
        {
            String strExt = Path.GetExtension(fileName);

            foreach (String s in SevenZipExtensions)
            {
                if (s == strExt.ToLower())
                {
                    return true;
                }
            }

            return false;
        }

        public static String[] getFilesList(String fileName) {
            String [] lstFiles = null;

            if (isRarFile(fileName))
                lstFiles = getRarFilesList(fileName);
            else if (isZipFile(fileName))
                lstFiles = getZipFilesList(fileName);
            else if (isTarFile(fileName))
                lstFiles = getTarFilesList(fileName);
            else if (isSevenZipFile(fileName))
                lstFiles = getSevenZipFilesList(fileName);

            return lstFiles;
        }

        private static String[] getRarFilesList(String fileName) {
           //String[] lstFiles;
           List<String> lstFiles = new List<String>();
           Unrar rar = new Unrar(fileName);

           try {
              rar.Open();
           } catch (Exception) {
              return null;
           }

           // Note: This isn't working as it should yet! To-do: Fix this someday... :(
           while (rar.ReadHeader()) {
              if (!rar.CurrentFile.ContinuedFromPrevious && !rar.CurrentFile.IsDirectory) {
                 lstFiles.Add(rar.CurrentFile.FileName);
              }
              rar.Skip();
           }

           // This wasn't working right because of multi-volume rar files being repeated
           //lstFiles = rar.ListFiles();

           rar.Close();

           if (lstFiles.Count < 1)
              return null;
           else
              return lstFiles.ToArray();
        }

        private static String[] getZipFilesList(String fileName) {
            List<String> lstFiles = new List<string>();
            ZipInputStream zis = null;
            ZipEntry ze;

            try {
                zis = new ZipInputStream(File.OpenRead(fileName));

                while ((ze = zis.GetNextEntry()) != null) {
                    if (ze.IsFile)
                        lstFiles.Add(ze.Name.Replace('/', '\\'));
                }
            } catch (Exception) {
                return null;
            } finally {
                if(zis != null)
                    zis.Close();
            }

            return lstFiles.ToArray();
        }

        private static String[] getTarFilesList(String fileName) {
            List<String> lstFiles = new List<string>();
            TarInputStream tis = null;
            TarEntry te;

            try {
                tis = new TarInputStream(File.OpenRead(fileName));

                while ((te = tis.GetNextEntry()) != null) {
                    if (!te.IsDirectory)
                        lstFiles.Add(te.Name.Replace('/', '\\'));
                }
            } catch (Exception) {
                return null;
            } finally {
                if (tis != null)
                    tis.Close();
            }

            return lstFiles.ToArray();
        }

        private static String[] getSevenZipFilesList(String fileName) {
           List<String> lstFiles = new List<string>();


           FileStream inStream = null;
           try {
              inStream = File.OpenRead(fileName);
           } catch (Exception) {
              return null;
           }

           SevenZip.ArchiveDatabaseEx archivedatabaseex;

           new SevenZip.SzIn().szArchiveOpen(inStream, out archivedatabaseex);

           for (int i = 0; i < archivedatabaseex.Database.NumFiles; i++) {
              SevenZip.FileItem file = archivedatabaseex.Database.Files[i];

              if (!file.IsDirectory) {
                 string fixed_name = file.Name.Substring(0, file.Name.Length - 1); //Fixed because the 7zip library returns the FileName with a space at the end
                 lstFiles.Add(fixed_name.Replace('/', '\\'));
              }
           }

           inStream.Close();

           return lstFiles.ToArray();
        }
    }
}
