/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using FirebirdSql.Data.FirebirdClient;
using System.Data;
using System.IO;
//using System.Windows.Forms;

namespace SDC.Net {
    public class DB {
        private FbConnection conn;

        public DB(String dbPath) {
            conn = new FbConnection("ServerType=1;User=SYSDBA;Password=masterkey;DataBase=" + dbPath + ";Dialect=3");
            //conn = new FbConnection("User=SYSDBA;Password=masterkey;DataBase=" + dbPath + ";Dialect=3");
        }

        public int addDisk(DiskData disk) {
            int diskid = getNextDiskID();

            String sql = "insert into Disks (DiskID, VolumeName, CatalogName," +
                         "SerialNumber, MediaType, Capacity, UsedSpace," +
                         "FreeSpace, AdditionalInfo) values (" +
                         diskid + "," + 
                         "'" + disk.getVolumeName().Replace("'", "''") + "'," +
                         "'" + disk.getCatalogName().Replace("'", "''") + "'," +
                         "'" + disk.getSerialNumber() + "'," +
                         "'" + disk.getMediaType().Replace("'", "''") + "'," +
                         disk.getCapacity() + "," +
                         disk.getUsedSpace() + "," +
                         disk.getFreeSpace() + "," +
                         "'" + disk.getAdditionalInfo().Replace("'", "''") + "')";


            FbCommand cmd = new FbCommand(sql, conn);
            
            cmd.ExecuteNonQuery();

            cmd.Dispose();

            return diskid;
        }

        private int getNextDiskID() {
            String sql = "select gen_id(gen_disks, 1) from rdb$database";

            FbCommand cmd = new FbCommand(sql, conn);

            int result = Convert.ToInt32(cmd.ExecuteScalar());

            cmd.Dispose();

            return result;
        }

        public void addFile(int diskID, FileData file) {
            String sql = "insert into Files (DiskID, ID, ParentID, Name, FileSize, DateModified, IsDirectory) values (" +
                         diskID + "," +
                         file.getID() + "," +
                         file.getParentID() + "," +
                         "'" + file.getName().Replace("'", "''") + "'," +
                         file.getSize() + "," +
                         "'" + file.getDateModified().ToString("MM-dd-yyyy hh:mm:ss") + "'," + 
                         (file.isDirectory()?1:0) + ")";

            FbCommand cmd = new FbCommand(sql, conn);

            cmd.ExecuteNonQuery();
            
            cmd.Dispose();
        }

        public bool diskExists(String serialNumber) {
            String sql = "select count(*) from Disks\n" +
                         "where SerialNumber = '" + serialNumber.Replace("'", "''") + "'";

            FbCommand cmd = new FbCommand(sql, conn);

            bool result = Convert.ToInt32(cmd.ExecuteScalar()) > 0 ? true : false;

            cmd.Dispose();

            return result;
        }
        
        public bool diskCatalogNameExists(String catalogName) {
            String sql = "select count(*) from Disks\n" +
                         "where CatalogName = '" + catalogName.Replace("'", "''") + "'";

            FbCommand cmd = new FbCommand(sql, conn);

            bool result = Convert.ToInt32(cmd.ExecuteScalar()) > 0 ? true : false;

            cmd.Dispose();

            return result;
        }
        
        public void deleteDisk(String serialNumber) {
            String sql = "delete from Disks\n" +
                         "where SerialNumber = '" + serialNumber + "'";

            FbCommand cmd = new FbCommand(sql, conn);

            cmd.ExecuteNonQuery();

            cmd.Dispose();
        }

        public List<DiskData> getDisksList() {
            String sql = "select * from Disks\n" +
                         "order by CatalogName";

            List<DiskData> lstDisks = new List<DiskData>();

            FbCommand cmd = new FbCommand(sql, conn);

            FbDataReader dr = cmd.ExecuteReader();

            while (dr.Read()) {
                lstDisks.Add(new DiskData(dr.GetInt32(dr.GetOrdinal("DiskID")), "", dr.GetString(dr.GetOrdinal("VolumeName")), dr.GetString(dr.GetOrdinal("SerialNumber")), dr.GetString(dr.GetOrdinal("MediaType")), dr.GetInt64(dr.GetOrdinal("Capacity")), dr.GetInt64(dr.GetOrdinal("UsedSpace")), dr.GetInt64(dr.GetOrdinal("FreeSpace")), dr.GetString(dr.GetOrdinal("CatalogName")), dr.GetString(dr.GetOrdinal("AdditionalInfo"))));
            }

            cmd.Dispose();
            
            return lstDisks;
        }

        public List<FileData> getFilesList(int diskID, int parentID) {
            String sql = "select * from files\n" +
                         "where diskid = " + diskID + "\n" +
                         "and parentid = " + parentID + "\n" +
                         "order by isdirectory desc, name asc\n";


            List<FileData> lstFiles = new List<FileData>();

            FbCommand cmd = new FbCommand(sql, conn);

            FbDataReader dr = cmd.ExecuteReader();

            while (dr.Read()) {
                FileData fd = new FileData(dr.GetInt32(dr.GetOrdinal("ID")), dr.GetInt32(dr.GetOrdinal("ParentID")), dr.GetInt32(dr.GetOrdinal("DiskID")), dr.GetString(dr.GetOrdinal("Name")), dr.GetInt64(dr.GetOrdinal("FileSize")), dr.GetDateTime(dr.GetOrdinal("DateModified")), dr.GetBoolean(dr.GetOrdinal("IsDirectory")));
                lstFiles.Add(fd);
            }

            cmd.Dispose();
            
            return lstFiles;
        }

        public bool hasChilds(int diskID, int fileID) {
            String sql = "select count(*) from files\n" +
                         "where diskid = " + diskID + "\n" +
                         "and parentid = " + fileID + "\n" +
                         "and isdirectory = 1";

            FbCommand cmd = new FbCommand(sql, conn);

            bool result = Convert.ToInt32(cmd.ExecuteScalar()) > 0 ? true : false;

            cmd.Dispose();

            return result;
        }

        public int getDiskID(String serialNumber) {
            String sql = "select diskid from disks\n" +
             "where serialnumber = '" + serialNumber + "'\n";

            FbCommand cmd = new FbCommand(sql, conn);

            int result = Convert.ToInt32(cmd.ExecuteScalar());

            cmd.Dispose();

            return result;
        }

        public List<FileData> srchFiles(List<String> names, List<int> disks, long size_start, long size_end, DateTime date_start, DateTime date_end) {
           List<FileData> lstFiles = new List<FileData>();
           String where_and = "where ";

           String sql = "select * from files\n";

           foreach (String s in names) {
              if (s.Trim().Length != 0) {
                 sql += where_and + "name containing '" + s.Replace("'", "''") + "'\n";
                 where_and = "and ";
              }
           }

           if (size_start >= 0 && size_end >= 0) {
              sql += where_and + "(filesize >= " + size_start + " and filesize <= " + size_end + ")\n";
              where_and = "and ";
           }

           if (date_start != DateTime.MinValue && date_end != DateTime.MinValue) {
              sql += where_and + "(DateModified between '" + date_start.ToString("yyyy-MM-dd 00:00:00") + "' and '" + date_end.ToString("yyyy-MM-dd 23:59:59") + "')\n";
              where_and = "and ";
           }

           if (disks.Count > 0) {
              sql += where_and + "diskid in (";

              for(int i = 0; i < disks.Count; i++) {
                 if (i != 0)
                    sql += ",";

                 sql += disks[i];
              }

              sql += ")";
           }

           sql += "order by isdirectory desc, name asc\n";

           FbCommand cmd = new FbCommand(sql, conn);

           FbDataReader dr = cmd.ExecuteReader();

           while (dr.Read()) {
              FileData fd = new FileData(dr.GetInt32(dr.GetOrdinal("ID")), dr.GetInt32(dr.GetOrdinal("ParentID")), dr.GetInt32(dr.GetOrdinal("DiskID")), dr.GetString(dr.GetOrdinal("Name")), dr.GetInt64(dr.GetOrdinal("FileSize")), dr.GetDateTime(dr.GetOrdinal("DateModified")), dr.GetBoolean(dr.GetOrdinal("IsDirectory")));
              lstFiles.Add(fd);
           }

           cmd.Dispose();

           return lstFiles;
        }

        public DiskData getDiskInfo(int diskID) {
            String sql = "select * from Disks\n" +
                         "where diskid = " + diskID;

            DiskData diskData = null;

            FbCommand cmd = new FbCommand(sql, conn);

            FbDataReader dr = cmd.ExecuteReader();

            if(dr.Read()) {
                diskData = new DiskData(dr.GetInt32(dr.GetOrdinal("DiskID")), "", dr.GetString(dr.GetOrdinal("VolumeName")), dr.GetString(dr.GetOrdinal("SerialNumber")), dr.GetString(dr.GetOrdinal("MediaType")), dr.GetInt64(dr.GetOrdinal("Capacity")), dr.GetInt64(dr.GetOrdinal("UsedSpace")), dr.GetInt64(dr.GetOrdinal("FreeSpace")), dr.GetString(dr.GetOrdinal("CatalogName")), dr.GetString(dr.GetOrdinal("AdditionalInfo")));
            }

            cmd.Dispose();

            return diskData;
        }

        private int getParentId(int diskID, int ID) {
            String sql = "select parentid from files\n" +
                         "where diskid = " + diskID + "\n" +
                         "and id = " + ID + "\n";

            FbCommand cmd = new FbCommand(sql, conn);

            int result = Convert.ToInt32(cmd.ExecuteScalar());

            cmd.Dispose();

            return result;
        }
        
        public String getFullFilePath(int diskID, int ID) {

            List<String> lstPath = new List<String>();
            String sql;
            String strResult = "" + Path.DirectorySeparatorChar;
            int parentID = getParentId(diskID, ID);
            FbCommand cmd;
            FbDataReader dr; 

            while (parentID != 0) {
                sql = "select parentid, name from files\n" +
                      "where id = " + parentID + "\n" +
                      "and diskid = " + diskID + "\n";

                cmd = new FbCommand(sql, conn);
                dr = cmd.ExecuteReader();
                dr.Read();
                parentID = dr.GetInt32(dr.GetOrdinal("ParentID"));
                lstPath.Add(dr.GetString(dr.GetOrdinal("Name")));
                cmd.Dispose();
            }

            for (int i = lstPath.Count - 1; i >= 0; i--) {
                strResult += lstPath[i];
                strResult += Path.DirectorySeparatorChar;
            }
            
            return strResult;
        }

        public List<int> getIDTree(int diskID, int ID) {
            List<int> lstIDTree = new List<int>();
            
            int parentID = getParentId(diskID, ID);

            lstIDTree.Add(ID);
            
            while (parentID != 0) {
                lstIDTree.Add(parentID);
                parentID = getParentId(diskID, parentID);
            }

            lstIDTree.Add(diskID);
            lstIDTree.Reverse();

            return lstIDTree;
        }

        public void updateDiskInfo(String serialNumber, String catalogName, String additionalInfo) {
            String sql = "update disks set \n" +
             "catalogName = '" + catalogName.Replace("'", "''") + "',\n" +
             "additionalinfo = '" + additionalInfo.Replace("'", "''") + "'\n" +
             "where serialnumber = '" + serialNumber + "'";

            FbCommand cmd = new FbCommand(sql, conn);

            cmd.ExecuteNonQuery();

            cmd.Dispose();
        }

        public void addCompressedFile(int diskID, int ID, int seq, String fileName, String pathName) {
            String sql = "insert into CompressedFiles (diskid, id, seq, name, directory) values (\n" +
                   diskID + "," +
                   ID + "," +
                   seq + "," +
                   "'" + fileName.Replace("'", "''") + "'," +
                   "'" + pathName.Replace("'", "''") + "')";


            
            FbCommand cmd = new FbCommand(sql, conn);

            cmd.ExecuteNonQuery();

            cmd.Dispose();
        }

        public List<FileData> srchCompressedFiles(List<String> names, List<int> disks) {
           List<FileData> lstFiles = new List<FileData>();
           String where_and = "where ";

           String sql = "select files.id, files.diskid, files.name as archive_name, files.parentid as archive_parentid, \n" +
                        "files.filesize as archive_size, files.datemodified as archive_datemodified, \n" +
                        "cf.name as file_name, cf.directory as file_directory\n" +
                        "from compressedfiles as cf\n" +
                        "left join files on (cf.id = files.id and cf.diskid = files.diskid)\n";

           foreach (String s in names) {
              sql += where_and + "cf.name containing '" + s.Replace("'", "''") + "'\n";
              where_and = "and ";
           }

           if (disks.Count > 0) {
              sql += where_and + "cf.diskid in (";

              for (int i = 0; i < disks.Count; i++) {
                 if (i != 0)
                    sql += ",";

                 sql += disks[i];
              }

              sql += ")";
           }

           FbCommand cmd = new FbCommand(sql, conn);

           FbDataReader dr = cmd.ExecuteReader();

           while (dr.Read()) {
              FileData fd = new FileData(dr.GetInt32(dr.GetOrdinal("ID")), dr.GetInt32(dr.GetOrdinal("archive_ParentID")), dr.GetInt32(dr.GetOrdinal("DiskID")), dr.GetString(dr.GetOrdinal("Archive_Name")) + "\\" + dr.GetString(dr.GetOrdinal("File_Name")), dr.GetInt64(dr.GetOrdinal("Archive_Size")), dr.GetDateTime(dr.GetOrdinal("Archive_DateModified")), false);
              lstFiles.Add(fd);
           }

           cmd.Dispose();

           return lstFiles;
        }

        public bool hasCompressedFiles(int diskID, int fileID) {
            String sql = "select count(*) from compressedfiles\n" +
                         "where diskid = " + diskID + "\n" +
                         "and id = " + fileID + "\n";

            FbCommand cmd = new FbCommand(sql, conn);

            bool result = Convert.ToInt32(cmd.ExecuteScalar()) > 0 ? true : false;

            cmd.Dispose();

            return result;
        }

        public DataTable getCompressedFiles(int diskID, int fileID) {
            String sql = "select Name, Directory from compressedfiles\n" +
                         "where diskid = " + diskID + "\n" +
                         "and id = " + fileID + "\n" +
                         "order by name\n";

            FbCommand cmd = new FbCommand(sql, conn);

            FbDataAdapter da = new FbDataAdapter(cmd);

            DataTable dt = new DataTable("CompressedFiles");

            da.Fill(dt);

            cmd.Dispose();

            return dt;
        }
        
        public void close() {
            conn.Close();
        }

        public void open() {
            conn.Open();
        }
    }
}
