/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Windows.Forms;

namespace SDC.Net {
    class DirectoryTreeBuilder {
        private String strRootDir;
        private Label lblStatus;
        private int diskID;
        private DB db;

        public DirectoryTreeBuilder(String strRootDir) {
            this.strRootDir = strRootDir;
            this.lblStatus = null;
            diskID = 0;
            db = null;
        }

        private void getDirectoryTreeList(Label lblStatus) {
            DirectoryInfo di = new DirectoryInfo(strRootDir);
            FileSystemInfo[] infos = di.GetFileSystemInfos();
            //List<FileData> lstDirectoryTree = new List<FileData>();
            int id = 0;

            this.lblStatus = lblStatus;

            buildDirectoryTreeList(infos, ref id);

            //return lstDirectoryTree;
        }

        public void getDirectoryTreeListAndSave(Label lblStatus, int diskID, DB db) {
            this.diskID = diskID;
            this.db = db;
            getDirectoryTreeList(lblStatus);
        }

        private void buildDirectoryTreeList(FileSystemInfo[] fsi, ref int nodeID) {
            if (fsi == null) {
                throw new ArgumentNullException("fsi");
            }

            //if (lstContents == null) {
            //    throw new ArgumentNullException("lstContents");
            //}

            int parentNode = nodeID;

            foreach (FileSystemInfo i in fsi) {
                if (i is DirectoryInfo) {
                    nodeID++;

                    FileData fd = new FileData(nodeID, parentNode, -1, i.Name, 0, i.CreationTime, true);
                    
                    //lstContents.Add(fd);

                    if(db != null) {
                        db.addFile(diskID, fd);
                    }
                    
                    try {
                        buildDirectoryTreeList(((DirectoryInfo)i).GetFileSystemInfos(), ref  nodeID);
                    } catch {
                        continue;
                    }

                } else if (i is FileInfo) {
                    nodeID++;
                    
                    if (lblStatus != null) {
                        lblStatus.Text = i.FullName;
                        lblStatus.Update();
                    }

                    FileData fd = new FileData(nodeID, parentNode, diskID, i.Name, ((FileInfo)i).Length, i.LastWriteTime, false);

                    //lstContents.Add(fd);

                    if(db != null) {
                        db.addFile(diskID, fd);
                        
                        if (CompressedFile.isCompressedFile(i.FullName))
                           try {
                              addCompressedFile(i.FullName, nodeID);
                           } catch(Exception) {
                              //MessageBox.Show(ex.Message);
                           }
                    }
                }
            }
        }

        private void addCompressedFile(String fileName, int nodeID) {
            String[] lstCompFiles = CompressedFile.getFilesList(fileName);
            int seq = 1;

            if (lstCompFiles == null) 
                return;

            foreach (String s in lstCompFiles) {
                db.addCompressedFile(diskID, nodeID, seq, Path.GetFileName(s), Path.GetDirectoryName(s));
                seq++;
            }
        }
    }
}
