/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.Text;
using System.Management;

namespace SDC.Net {
    public class DiskData {
        private int diskID;
        private String driveName;
        private String volumeName;
        private String catalogName;
        private String serialNumber;
        private String mediaType;
        private long capacity;
        private long usedSpace;
        private long freeSpace;
        private String additionalInfo;

        public DiskData(int diskID, String driveName, String volumeName, String serialNumber, String mediaType, long capacity, long usedSpace, long freeSpace, String catalogName, String additionalInfo) {
            this.diskID = diskID;
            this.driveName = driveName;
            this.volumeName = volumeName;
            this.serialNumber = serialNumber;
            this.mediaType = mediaType;
            this.capacity = capacity;
            this.usedSpace = usedSpace;
            this.freeSpace = freeSpace;
            this.catalogName = catalogName;
            this.additionalInfo = additionalInfo;
        }
        
        public DiskData(ManagementObject mo, String catalogName, String additionalInfo) {
            String tmp;
            
            // Set ManagementObject independent info
            this.catalogName = catalogName;
            this.additionalInfo = additionalInfo;
            this.diskID = -1; // This only makes sense when getting from catalog database

            // Set ManagementObject dependent info
            driveName = Convert.ToString(mo["Name"]);
            volumeName = Convert.ToString(mo["VolumeName"]);
            
            tmp = Convert.ToString(mo["VolumeSerialNumber"]);
            if (tmp.Length > 0) {
                setSerialNumber(long.Parse(tmp, System.Globalization.NumberStyles.HexNumber).ToString("x").ToUpper().PadLeft(8, '0'));
            }

            mediaType = Convert.ToString(mo["Description"]);

            tmp = Convert.ToString(mo["Size"]);
            if (tmp.Length > 0) {
                capacity = long.Parse(tmp);
            }

            tmp = Convert.ToString(mo["FreeSpace"]);
            if (tmp.Length > 0) {
                freeSpace = long.Parse(tmp);
            }

            usedSpace = capacity - freeSpace;
        }

        public String getDriveName() {
            return driveName;
        }

        public String getVolumeName() {
            return volumeName;
        }

        public String getCatalogName() {
            return catalogName;
        }

        private void setSerialNumber(String serialNumber) {
            this.serialNumber = serialNumber.Substring(0, 4) + "-" + serialNumber.Substring(4, 4);
        }
        
        public String getSerialNumber() {
            if (serialNumber != null)
                return serialNumber;
            else
                return "";
        }

        public String getMediaType() {
            return mediaType;
        }

        public long getCapacity() {
            return capacity;
        }

        public long getFreeSpace() {
            return freeSpace;
        }

        public long getUsedSpace() {
            return usedSpace;
        }

        public String getAdditionalInfo() {
            return additionalInfo;
        }

        public int getDiskID() {
            return diskID;
        }

        public void setCatalogName(String catalogName) {
            this.catalogName = catalogName;
        }

        public void setAdditionalInfo(String additionalInfo) {
            this.additionalInfo = additionalInfo;
        }

        public static List<DiskData> getDiskDataList() {
            ManagementObjectSearcher mos = new ManagementObjectSearcher(new SelectQuery("select * from Win32_LogicalDisk"));
            ManagementObjectCollection moc = mos.Get();

            List<DiskData> lstDiskData = new List<DiskData>();

            foreach (ManagementObject mo in moc) {
                lstDiskData.Add(new DiskData(mo, "", ""));
            }

            return lstDiskData;
        }

        public override string ToString() {
           return catalogName;
        }
    }
}
