/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SDC.Net {
    public partial class FrmAddDisk : Form {
        private DB db;
        private List<DiskData> lstDisks;
        private bool wasCatalogModified; // Tells if catalog was modified, for disks list refreshing on main screen

        public FrmAddDisk(DB db) {
            InitializeComponent();
            this.db = db;
            wasCatalogModified = false;
        }

        private void FrmAddDisk_Load(object sender, EventArgs e) {
            loadAvailableDisks();
        }

        private void loadAvailableDisks() {
            this.Cursor = Cursors.WaitCursor;

            lstDisks = DiskData.getDiskDataList();

            dgvDisks.Rows.Clear();

            foreach (DiskData dd in lstDisks) {
                dgvDisks.Rows.Add(new Object[] { dd.getDriveName(), dd.getVolumeName(), dd.getSerialNumber(), dd.getMediaType(), FileData.getFormatedFileSize(dd.getCapacity()), FileData.getFormatedFileSize(dd.getUsedSpace()), FileData.getFormatedFileSize(dd.getFreeSpace()) });
            }

            this.Cursor = Cursors.Default;
        }

        private void btnClose_Click(object sender, EventArgs e) {
            Close();
        }

        private void btnRefresh_Click(object sender, EventArgs e) {
            loadAvailableDisks();
        }

        private void btnAdd_Click(object sender, EventArgs e) {
            String strDrvLetter = dgvDisks.SelectedRows[0].Cells[0].Value.ToString();
            DriveInfo di = new DriveInfo(strDrvLetter);

            if (!di.IsReady) {
                MessageBox.Show(this, "Drive " + strDrvLetter + " is not ready!", "Error");
                return;
            }

            // Update the disks list to make sure the next screen has updated disk data
            lstDisks = DiskData.getDiskDataList();
            
            FrmAddDiskInfo f = new FrmAddDiskInfo(lstDisks[dgvDisks.SelectedRows[0].Index], db, true);
            
            f.ShowDialog(this);

            if (f.DialogResult == DialogResult.OK)
                wasCatalogModified = true;
            
            f.Dispose();
        }

        private void FrmAddDisk_FormClosing(object sender, FormClosingEventArgs e) {
            if (wasCatalogModified)
                this.DialogResult = DialogResult.OK;
        }
    }
}
