/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SDC.Net {
    public partial class FrmAddDiskInfo : Form {
        private DiskData dd;
        private DB db;
        private DirectoryTreeBuilder dtb;
        private int diskid;
        private bool wasCatalogModified; // Tells if catalog was modified, for disks list refreshing on main screen
        private bool searchMode; // This forms has 2 purposes: Initiate a disk search and modify disk info.
        
        public FrmAddDiskInfo(DiskData dd, DB db, bool searchMode) {
            InitializeComponent();
            this.dd = dd;
            this.db = db;
            wasCatalogModified = false;
            this.searchMode = searchMode;
        }

        private void btnClose_Click(object sender, EventArgs e) {
            Close();
        }

        private void FrmAddDiskInfo_Load(object sender, EventArgs e) {
            btnOK.Visible = searchMode;
            btnSave.Visible = !searchMode;
            
            if(searchMode)
                txtCatalogName.Text = dd.getVolumeName();
            else
                txtCatalogName.Text = dd.getCatalogName();
            txtVolumeName.Text = dd.getVolumeName();
            txtSerialNumber.Text = dd.getSerialNumber();
            txtMediaType.Text = dd.getMediaType();
            rtAdditionalInfo.Text = dd.getAdditionalInfo();


            txtCapacity.Text = FileData.getFormatedFileSize(dd.getCapacity());
            txtUsedSpace.Text = FileData.getFormatedFileSize(dd.getUsedSpace());
            txtFreeSpace.Text = FileData.getFormatedFileSize(dd.getFreeSpace());
        }

        private void btnOK_Click(object sender, EventArgs e) {
            if (txtCatalogName.Text.Trim().Length == 0) {
                MessageBox.Show(this, "You must specify a Catalog Name for this disk!", "Warning!");
                txtCatalogName.Focus();
                return;
            }

            DriveInfo di = new DriveInfo(dd.getDriveName());

            if (!di.IsReady) {
                MessageBox.Show(this, "Drive " + dd.getDriveName() + " is not ready!", "Error");
                return;
            }

            if (db.diskExists(dd.getSerialNumber())) {
                if(MessageBox.Show(this, "There is already a disk with serial number " + dd.getSerialNumber() + " in the catalog.\nDo you want to replace its data?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                    return;
            }
            
            if (db.diskCatalogNameExists(txtCatalogName.Text)) {
            	if(MessageBox.Show(this, "There is already a disk with Catalog Name '" + txtCatalogName.Text + "' in the catalog.\nDo you want to proceed anyway?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes) {
            		txtCatalogName.Focus();
            		return;
            	}
            }

            if (MessageBox.Show(this, "Start disk cataloging process?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button1) != DialogResult.Yes)
                return;

            gbDiskInfo.Enabled = false;
            btnOK.Enabled = false;
            btnClose.Enabled = false;
            this.Cursor = Cursors.WaitCursor;

            lblStatus.Text = "Deleting disk previous data...";
            lblStatus.Update();

            db.deleteDisk(dd.getSerialNumber());

            dd.setCatalogName(txtCatalogName.Text);
            dd.setAdditionalInfo(rtAdditionalInfo.Text);

            dtb = new DirectoryTreeBuilder(dd.getDriveName() + Path.DirectorySeparatorChar);

            diskid = db.addDisk(dd);


            bgw.RunWorkerAsync();
            //dtb.getDirectoryTreeListAndSave(lblStatus, diskid, db);
            //btnOK.Enabled = true;
            //btnClose.Enabled = true;
            //this.Cursor = Cursors.Default;

        }

        private void bgw_DoWork(object sender, DoWorkEventArgs e) {
            dtb.getDirectoryTreeListAndSave(lblStatus, diskid, db);
        }

        private void bgw_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
            btnOK.Enabled = true;
            btnClose.Enabled = true;
            this.Cursor = Cursors.Default;
            gbDiskInfo.Enabled = true;

            wasCatalogModified = true;

            lblStatus.Text = "Ready!";

            MessageBox.Show(this, "Disk cataloging proccess finished!", "Message");
            
            Close();
        }

        private void FrmAddDiskInfo_FormClosing(object sender, FormClosingEventArgs e) {
            if (bgw.IsBusy) {
                e.Cancel = true;
                return;
            } else
                e.Cancel = false;

            if (wasCatalogModified) {
                System.GC.Collect();
                this.DialogResult = DialogResult.OK;
            }
        }

        private void btnSave_Click(object sender, EventArgs e) {
            if (txtCatalogName.Text.Trim().Length == 0) {
                MessageBox.Show(this, "You must specify a Catalog Name for this disk!", "Warning!");
                txtCatalogName.Focus();
                return;
            }

            db.updateDiskInfo(dd.getSerialNumber(), txtCatalogName.Text, rtAdditionalInfo.Text);
            this.DialogResult = DialogResult.OK;
            Close();
        }
    }
}
