/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;

namespace SDC.Net {
    public partial class FrmMain : Form {
        private DB db;
        private ListViewColumnSorter lvwColumnSorter;

        public FrmMain(DB db) {
           InitializeComponent();

           CheckForIllegalCrossThreadCalls = false;

           this.Text += String.Format(" - Version {0}", Assembly.GetExecutingAssembly().GetName().Version.ToString());

           this.db = db;

           lvwColumnSorter = new ListViewColumnSorter();
           this.lvData.ListViewItemSorter = lvwColumnSorter;
        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
            Close();
        }

        private void FrmMain_Load(object sender, EventArgs e) {
            updateTreeView();
        }

        private void updateTreeView() {
            List<DiskData> lstDisks = db.getDisksList();

            tvData.Nodes.Clear();
            lvData.Items.Clear();
            
            foreach (DiskData dd in lstDisks) {
                TreeNode n = new TreeNode(dd.getCatalogName());
                n.Tag = dd;
                n.ImageIndex = 2;
                n.SelectedImageIndex = 2;
                n.Nodes.Add(""); // Dummy Node

                tvData.Nodes.Add(n);
            }

            tsslblPath.Text = "";
        }

        private void tvData_BeforeExpand(object sender, TreeViewCancelEventArgs e) {
            if (e.Node.Nodes.Count != 1 || e.Node.Nodes[0].Text != "") return;

            if (e.Node.Level == 0) { // Disks
                int diskID = db.getDiskID(((DiskData)e.Node.Tag).getSerialNumber());
                e.Node.Nodes.Clear();

                foreach (FileData fd in db.getFilesList(diskID, 0)) {
                    if (!fd.isDirectory()) continue;

                    TreeNode tn = new TreeNode(fd.getName());
                    tn.Tag = fd;

                    if (db.hasChilds(diskID, fd.getID()))
                        tn.Nodes.Add("");

                    e.Node.Nodes.Add(tn);
                }

            } else { //Files and directories
                e.Node.Nodes.Clear();

                foreach (FileData fd in db.getFilesList(((FileData)e.Node.Tag).getDiskID(), ((FileData)e.Node.Tag).getID())) {
                    if (!fd.isDirectory()) continue;

                    TreeNode tn = new TreeNode(fd.getName());
                    tn.Tag = fd;

                    if (db.hasChilds(((FileData)e.Node.Tag).getDiskID(), fd.getID()))
                        tn.Nodes.Add("");

                    e.Node.Nodes.Add(tn);
                }
            }
 
        }

        private void addDiskToCatalogToolStripMenuItem_Click(object sender, EventArgs e) {
            FrmAddDisk f = new FrmAddDisk(db);
            f.ShowDialog(this);

            if(f.DialogResult == DialogResult.OK)
                updateTreeView();

            f.Dispose();
        }


        private void tvData_AfterSelect(object sender, TreeViewEventArgs e) {
            int diskID;
            List<FileData> lstFiles = null;
            
            if (tvData.SelectedNode.Level == 0) { // Disk
                diskID = ((DiskData)tvData.SelectedNode.Tag).getDiskID();
                lstFiles = db.getFilesList(diskID, 0);
            } else {
                diskID = ((FileData)tvData.SelectedNode.Tag).getDiskID();
                lstFiles = db.getFilesList(diskID, ((FileData)tvData.SelectedNode.Tag).getID());
            }

            

            lvData.Items.Clear();
            foreach (FileData fd in lstFiles) {
                ListViewItem lvi = new ListViewItem(new String[] { fd.getName(), fd.isDirectory()?"":FileData.getFormatedFileSize(fd.getSize()), fd.getDateModified().ToString("g") }, fd.isDirectory() ? 1 : 0);

                lvi.Tag = fd;
                
                lvData.Items.Add(lvi);
            }


            tsslblPath.Text = "Current folder: ";
            if (tvData.SelectedNode.Level == 0)
                tsslblPath.Text += ((DiskData)tvData.SelectedNode.Tag).getCatalogName() + "\\";
            else
                tsslblPath.Text += db.getDiskInfo(((FileData)tvData.SelectedNode.Tag).getDiskID()).getCatalogName() + 
                                  db.getFullFilePath(((FileData)tvData.SelectedNode.Tag).getDiskID(), ((FileData)tvData.SelectedNode.Tag).getID()) + ((FileData)tvData.SelectedNode.Tag).getName() + "\\";
        }

        private void FrmMain_FormClosing(object sender, FormClosingEventArgs e) {
            if (MessageBox.Show(this, "Are you sure you want to close the application?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) == DialogResult.Yes) {
                e.Cancel = false;
                db.close();
            } else {
                e.Cancel = true;
            }
        }

        private void lvData_DoubleClick(object sender, EventArgs e) {
           if (lvData.SelectedItems.Count == 0) return;

           FileData fd = (FileData)lvData.SelectedItems[0].Tag;

           if (fd.isDirectory()) {
              tvData.SelectedNode.Expand();
              foreach (TreeNode tn in tvData.SelectedNode.Nodes) {
                 if (((FileData)tn.Tag).getID() == fd.getID()) {
                    tvData.SelectedNode = tn;
                    break;
                 }
              }
           }

           if (db.hasCompressedFiles(fd.getDiskID(), fd.getID())) {
              FrmViewCompressedFiles f = new FrmViewCompressedFiles(db, fd.getDiskID(), fd.getID());
              f.ShowDialog(this);
              f.Dispose();
           }
        }

        private void toolStripMenuItem2_Click(object sender, EventArgs e) {
            List<int> lstID;
            FrmSearch f = new FrmSearch(db);
            f.ShowDialog(this);

            if ((lstID = f.getIDTreeList()) != null)
                lookUpFile(lstID);

            f.Dispose();
        }

        private void lookUpFile(List<int> lstID) {
            // First, we look in the first level for the DiskID
            foreach (TreeNode tn in tvData.Nodes) {
                if (((DiskData)tn.Tag).getDiskID() == lstID[0]) {
                    tvData.SelectedNode = tn;
                    tvData.SelectedNode.Expand();
                    tvData.Select();
                    break;
                }
            }

            // We go down the selected disk and expand-sub folders
            for (int i = 1; i < lstID.Count - 1; i++) {
                foreach (TreeNode tn in tvData.SelectedNode.Nodes) {
                    if (((FileData)tn.Tag).getID() == lstID[i]) {
                        tvData.SelectedNode = tn;
                        tvData.SelectedNode.Expand();
                        break;
                    }
                }
            }

            // Finally, we select the file in the ListView control
            foreach (ListViewItem lvi in lvData.Items) {
                if (((FileData)lvi.Tag).getID() == lstID[lstID.Count - 1]) {
                    lvData.Items[lvi.Index].Selected = true;
                    lvData.Select();
                    lvData.SelectedItems[0].EnsureVisible();
                    break;
                }
            }
        }

        private void aboutThisSoftwareToolStripMenuItem_Click(object sender, EventArgs e) {
            FrmAbout f = new FrmAbout();
            f.ShowDialog(this);
            f.Dispose();
        }

        private void toolStripMenuItem3_Click(object sender, EventArgs e) {
            DiskData dd;
            
            if (tvData.Nodes.Count == 0) {
                MessageBox.Show(this, "There aren't disks to be deleted!", "Warning!");
                return;
            }

            if (tvData.SelectedNode == null) {
                MessageBox.Show(this, "You must select a disk first!", "Warning!");
                return;
            }

            if(tvData.SelectedNode.Level == 0) 
                dd = (DiskData)tvData.SelectedNode.Tag;
            else
                dd = db.getDiskInfo(((FileData)tvData.SelectedNode.Tag).getDiskID());

            if (MessageBox.Show(this, "Do you really want to remove disk '" + dd.getCatalogName() + "', serial number " + dd.getSerialNumber() + " from the catalog?", "Confirmation", MessageBoxButtons.YesNo, MessageBoxIcon.Question, MessageBoxDefaultButton.Button2) != DialogResult.Yes)
                return;

            db.deleteDisk(dd.getSerialNumber());

            updateTreeView();
        }

        private void toolStripMenuItem4_Click(object sender, EventArgs e) {
            DiskData dd;
            
            if (tvData.Nodes.Count == 0) {
                MessageBox.Show(this, "There aren't disks to be modified!", "Warning!");
                return;
            }

            if (tvData.SelectedNode == null) {
                MessageBox.Show(this, "You must select a disk first!", "Warning!");
                return;
            }

            if(tvData.SelectedNode.Level == 0) 
                dd = (DiskData)tvData.SelectedNode.Tag;
            else
                dd = db.getDiskInfo(((FileData)tvData.SelectedNode.Tag).getDiskID());


            FrmAddDiskInfo f = new FrmAddDiskInfo(dd, db, false);
            f.ShowDialog(this);
            
            if (f.DialogResult == DialogResult.OK)
                updateTreeView();
            
            f.Dispose();
        }

        private void mnuLV_Opening(object sender, CancelEventArgs e) {
            if(lvData.SelectedItems.Count == 0) return;

            FileData fd = (FileData)lvData.SelectedItems[0].Tag;

            openFolderToolStripMenuItem.Enabled = fd.isDirectory();
            viewCompressedFilesToolStripMenuItem.Enabled = db.hasCompressedFiles(fd.getDiskID(), fd.getID());
        }

        private void toolStripButton5_Click(object sender, EventArgs e) {
            if (tvData.SelectedNode == null) return;

            TreeNode up = tvData.SelectedNode.Parent;

            if (up != null)
                tvData.SelectedNode = up;
        }

        private void lvData_ColumnClick(object sender, ColumnClickEventArgs e) {
           // Determine if clicked column is already the column that is being sorted.
           if (e.Column == lvwColumnSorter.SortColumn) {
              // Reverse the current sort direction for this column.
              if (lvwColumnSorter.Order == SortOrder.Ascending) {
                 lvwColumnSorter.Order = SortOrder.Descending;
              } else {
                 lvwColumnSorter.Order = SortOrder.Ascending;
              }
           } else {
              // Set the column number that is to be sorted; default to ascending.
              lvwColumnSorter.SortColumn = e.Column;
              lvwColumnSorter.Order = SortOrder.Ascending;
           }

           // Perform the sort with these new sort options.
           this.lvData.Sort();
        }
    }
}
