/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Web;

namespace SDC.Net {
    public partial class FrmSearch : Form {
        private DB db;
        private List<int> lstID;
        private List<FileData> lstFiles, lstZipFiles;
        private long size_start, size_end;
        private bool cancelSearch;


        public FrmSearch(DB db) {
            InitializeComponent();
            this.db = db;
            this.lstID = null;
        }

        public List<int> getIDTreeList() {
            return lstID;
        }

        private void chbDateRange_CheckedChanged(object sender, EventArgs e) {
            if (chbDateRange.Checked) {
                dtpStart.Enabled = dtpEnd.Enabled = true;
                dtpStart.Focus();
            } else {
                dtpStart.Enabled = dtpEnd.Enabled = false;
            }
        }

        private void chbSizeRange_CheckedChanged(object sender, EventArgs e) {
            if (chbSizeRange.Checked) {
                mskStart.Enabled = mskEnd.Enabled = true;
                mskStart.Focus();
            } else {
                mskStart.Enabled = mskEnd.Enabled = false;
            }

        }

        private void btnClose_Click(object sender, EventArgs e) {
            Close();
        }

        private void btnSearch_Click(object sender, EventArgs e) {
           if (bgwSearch.IsBusy) {
              cancelSearch = true;
              return;
           }

           cancelSearch = false;

           size_start = size_end = -1;

           if (chbDateRange.Checked) {
              if (dtpStart.Value > dtpEnd.Value) {
                 MessageBox.Show(this, "Start date can't be bigger than end date.", "Warning");
                 dtpStart.Focus();
                 return;
              }
           }

           if (chbSizeRange.Checked) {
              try {
                 size_start = Convert.ToInt64(mskStart.Text.Trim());
                 size_end = Convert.ToInt64(mskEnd.Text.Trim());
              } catch (Exception) {
                 MessageBox.Show(this, "Please check size limits for invalid data!", "Warning");
                 mskStart.Focus();
                 return;
              }

              if (size_start > size_end) {
                 MessageBox.Show(this, "Start size can't be bigger than end size!", "Warning");
                 mskStart.Focus();
                 return;
              }
           }

           if (cklstbxDisks.CheckedItems.Count < 1) {
              MessageBox.Show(this, "You must select at least one disk from the list!", "Warning");
              cklstbxDisks.Focus();
              return;
           }

           this.Cursor = Cursors.WaitCursor;
           gbDateRange.Enabled = false;
           gbSizeRange.Enabled = false;
           txtName.Enabled = false;
           btnClose.Enabled = false;
           btnExport.Enabled = false;
           chbCompressedFiles.Enabled = false;
           btnSearch.Text = "Cancel";

           lblStatus.Text = "Querying database...";

           bgwSearch.RunWorkerAsync();
        }

        private void lvResults_DoubleClick(object sender, EventArgs e) {
            if (lvResults.SelectedItems.Count == 0) return;

            lstID = db.getIDTree(((FileData)lvResults.SelectedItems[0].Tag).getDiskID(), ((FileData)lvResults.SelectedItems[0].Tag).getID());

            Close();
        }

        private List<String> mySplit(String str, char separator) {
            String token = "";
            List<String> lstData = new List<String>();
            bool inside_quotes = false;

            for (int i = 0; i < str.Length; i++) {
                if(str[i] == '"' || str[i] == '\'') 
                    inside_quotes = !inside_quotes;
                
                if(str[i] == separator && !inside_quotes){
                    lstData.Add(token);
                    token = "";
                } else {
                    if(str[i] != '"' && str[i] != '\'')
                        token += str[i];
                }

            }

            lstData.Add(token);

            return lstData;
        }

        private void FrmSearch_Load(object sender, EventArgs e) {
           dtpStart.Value = dtpEnd.Value = DateTime.Now;

           // Disks checkbox
           List<DiskData> disks = db.getDisksList();
           
           foreach (DiskData d in disks) {
              cklstbxDisks.Items.Add(d, true);
           }
        }

        private void bgwSearch_DoWork(object sender, DoWorkEventArgs e) {
           List<int> disks = new List<int>();
           int count = 0, totalfiles;

           foreach (DiskData ci in cklstbxDisks.CheckedItems) {
              disks.Add(ci.getDiskID());
           }

           lstFiles = db.srchFiles(mySplit(txtName.Text, ' '), disks, chbSizeRange.Checked ? size_start : -1, chbSizeRange.Checked ? size_end : -1, chbDateRange.Checked ? dtpStart.Value : DateTime.MinValue, chbDateRange.Checked ? dtpEnd.Value : DateTime.MinValue);

           totalfiles = lstFiles.Count;

           if (chbCompressedFiles.Checked) {
              lstZipFiles = db.srchCompressedFiles(mySplit(txtName.Text, ' '), disks);
              totalfiles += lstZipFiles.Count;
           }

           lvResults.BeginUpdate();

           lvResults.Items.Clear();

           foreach (FileData fd in lstFiles) {
              ListViewItem lvi = new ListViewItem(new String[] { fd.getName(), db.getDiskInfo(fd.getDiskID()).getCatalogName(), db.getFullFilePath(fd.getDiskID(), fd.getID()) }, fd.isDirectory() ? 1 : 0);

              lvi.Tag = fd;

              lvResults.Items.Add(lvi);

              lblStatus.Text = "Filling results list... " + (count * 100 / totalfiles) + "%";

              count++;

              if (cancelSearch)
                 return;
           }

           if (!chbCompressedFiles.Checked)
              return;

           foreach (FileData fd in lstZipFiles) {
              ListViewItem lvi = new ListViewItem(new String[] { fd.getName().Split(new char[] { '\\' }).GetValue(1).ToString(), db.getDiskInfo(fd.getDiskID()).getCatalogName(), db.getFullFilePath(fd.getDiskID(), fd.getID()) + fd.getName().Split(new char[] { '\\' }).GetValue(0) }, fd.isDirectory() ? 1 : 0);

              lvi.Tag = fd;

              lvResults.Items.Add(lvi);

              lblStatus.Text = "Filling results list... " + (count * 100 / totalfiles) + "%";

              count++;

              if (cancelSearch)
                 return;
           }
        }

        private void bgwSearch_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e) {
            this.Cursor = Cursors.Default;
            gbDateRange.Enabled = true;
            gbSizeRange.Enabled = true;
            txtName.Enabled = true;
            btnClose.Enabled = true;
            btnExport.Enabled = true;
            chbCompressedFiles.Enabled = true;
            //btnSearch.Enabled = true;
            btnSearch.Text = "&Search";

            lvResults.EndUpdate();

            lblStatus.Text = lvResults.Items.Count + " item(s) found!";

            txtName.Focus();
        }

        private void FrmSearch_FormClosing(object sender, FormClosingEventArgs e) {
            e.Cancel = bgwSearch.IsBusy;
        }


        private void btnExport_Click(object sender, EventArgs e) {
        	if (sfdExportResults.ShowDialog() == DialogResult.OK) {
                using (TextWriter writer = new StreamWriter(sfdExportResults.FileName)) {
        			string strTitle = "Search results for \"" + HttpUtility.HtmlEncode(txtName.Text) + "\"";
                    writer.WriteLine("<HTML>");
                    writer.WriteLine("<HEAD><TITLE>" + strTitle + "</TITLE></HEAD>");
                    writer.WriteLine("<BODY style='font-family: Tahoma'>");
                    writer.WriteLine("<H1 align=center>" + strTitle + "</H1>");
                    writer.WriteLine("<BR>");

                    writer.WriteLine("<TABLE align=center style='width: 1024px; border-style: solid; border-size: 1px; border-color: #000000'>");
                    writer.WriteLine("<TR style='font-weight: bold; background-color: #000000; color: #FFFFFF'>");
                    writer.WriteLine("<TD style='width:30%'>File</TD>");
                    writer.WriteLine("<TD style='width:20%'>Disk</TD>");
                    writer.WriteLine("<TD style='width:50%'>Path</TD>");
                    writer.WriteLine("</TR>");

                    for (int i = 0; i < lvResults.Items.Count; i++) {
                    	writer.WriteLine("<TR style='background-color: " +((i%2 == 0)?"#CCCCCC":"#FFFFFF")+ "'>");
                        writer.WriteLine("<TD>" + HttpUtility.HtmlEncode(lvResults.Items[i].SubItems[0].Text) + "</TD>");
                        writer.WriteLine("<TD>" + HttpUtility.HtmlEncode(lvResults.Items[i].SubItems[1].Text) + "</TD>");
                        writer.WriteLine("<TD>" + HttpUtility.HtmlEncode(lvResults.Items[i].SubItems[2].Text) + "</TD>");
                        writer.WriteLine("</TR>");
                    }

                    writer.WriteLine("</TABLE>");

                    writer.WriteLine("<BR>");
                    writer.WriteLine("<DIV align=center>Auto-generated by Simple Disk Catalog .NET on " + DateTime.Now.ToString("G") + "</DIV>");

                    writer.WriteLine("</BODY>");
                    writer.WriteLine("</HTML>");
                }
            }
        }

        private void button1_Click(object sender, EventArgs e) {
           for (int i = 0; i < cklstbxDisks.Items.Count; i++)
              cklstbxDisks.SetItemChecked(i, true);
        }

        private void button2_Click(object sender, EventArgs e) {
           for (int i = 0; i < cklstbxDisks.Items.Count; i++)
              cklstbxDisks.SetItemChecked(i, !cklstbxDisks.GetItemChecked(i));
        }
    }
}
