﻿/*
   Simple Disk Catalog .Net - The OpenSource Disk Catalog Software now for .Net!
   Author: Bruno Freitas - 10/2007
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2005 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace SDC.Net {
   public class ListViewColumnSorter : IComparer {
      /// <summary>
      /// Specifies the column to be sorted
      /// </summary>
      private int ColumnToSort;
      /// <summary>
      /// Specifies the order in which to sort (i.e. 'Ascending').
      /// </summary>
      private SortOrder OrderOfSort;
      /// <summary>
      /// Case insensitive comparer object
      /// </summary>
      private CaseInsensitiveComparer ObjectCompare;

      /// <summary>
      /// Class constructor.  Initializes various elements
      /// </summary>
      public ListViewColumnSorter() {
         // Initialize the column to '0'
         ColumnToSort = 0;

         // Initialize the sort order to 'none'
         OrderOfSort = SortOrder.None;

         // Initialize the CaseInsensitiveComparer object
         ObjectCompare = new CaseInsensitiveComparer();
      }

      /// <summary>
      /// This method is inherited from the IComparer interface.  It compares the two objects passed using a case insensitive comparison.
      /// </summary>
      /// <param name="x">First object to be compared</param>
      /// <param name="y">Second object to be compared</param>
      /// <returns>The result of the comparison. "0" if equal, negative if 'x' is less than 'y' and positive if 'x' is greater than 'y'</returns>
      public int Compare(object x, object y) {
         int compareResult = 0;
         ListViewItem listviewX, listviewY;

         // Cast the objects to be compared to ListViewItem objects
         listviewX = (ListViewItem)x;
         listviewY = (ListViewItem)y;

         // Compare the two items
         switch (ColumnToSort) {
            case 0: 
               compareResult = ObjectCompare.Compare(((FileData)listviewX.Tag).getName(), ((FileData)listviewY.Tag).getName());
               break;
            case 1:
               compareResult = ObjectCompare.Compare(((FileData)listviewX.Tag).getSize(), ((FileData)listviewY.Tag).getSize());
               break;
            case 2:
               compareResult = ObjectCompare.Compare(((FileData)listviewX.Tag).getDateModified(), ((FileData)listviewY.Tag).getDateModified());
               break;
         }

         //compareResult = ObjectCompare.Compare(listviewX.SubItems[ColumnToSort].Text, listviewY.SubItems[ColumnToSort].Text);

         // Calculate correct return value based on object comparison
         if (OrderOfSort == SortOrder.Ascending) {
            // Ascending sort is selected, return normal result of compare operation
            return compareResult;
         } else if (OrderOfSort == SortOrder.Descending) {
            // Descending sort is selected, return negative result of compare operation
            return (-compareResult);
         } else {
            // Return '0' to indicate they are equal
            return 0;
         }
      }

      /// <summary>
      /// Gets or sets the number of the column to which to apply the sorting operation (Defaults to '0').
      /// </summary>
      public int SortColumn {
         set {
            ColumnToSort = value;
         }
         get {
            return ColumnToSort;
         }
      }

      /// <summary>
      /// Gets or sets the order of sorting to apply (for example, 'Ascending' or 'Descending').
      /// </summary>
      public SortOrder Order {
         set {
            OrderOfSort = value;
         }
         get {
            return OrderOfSort;
         }
      }
   }
}
